@ECHO OFF
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Programa:		Batch para carga de sincronismo de objetos na SDM
:: Modulo:			z_loader_sdm.cmd
:: Objetivo:		Sincronizar informacoes em objetos da ferramenta CA SDM baseado em um arquivo de dados.
:: Autor:			Daniel Becker Bighelini
:: Criado em:		30/11/2015
:: Modificado por:	Daniel Becker Bighelini
:: Modificado em:	20/02/2017
:: Verso:			1.02 (pt-br)
:: Funcionamento:	
::                  1) Caso existam arquivos .CSV na pasta atual do BATCH, esta rotina eh executada;
::					2) A rotina dispara a funcao SPEL 'z_loader_sdm' armazenada no arquivo 'z_loader_sdm.spl';
::					3) A partir de definicoes de configuracao no respectivo arquivo .properties relacionado
::					   ao arquivo .CSV que esta sendo processado, o sincronismo com a SDM  executado;
::					4) Os arquivos .CSV so processados em ordem alfabetica crescente;
::					5) Todo arquivo .CSV processado eh movido para uma pasta predefinida de processamento;
::					6) Quando nao existirem mais arquivos .CSV para serem processados, a rotina eh encerrada.
::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
:: Habilita a expansao das variaveis de ambiente
SETLOCAL ENABLEDELAYEDEXPANSION

:: Salva pasta atual
PUSHD "%~dp0"

:inicio
:: Define variaveis globais
CALL:atualiza_data_hora
SET batauthor=Daniel Becker Bighelini
SET batfunction=BATCH Z_LOADER_SDM
SET batversion=1.02
SET batdescription=SINCRONISMO DE INFORMACOES NA SDM BASEADO EM ARQUIVOS DE DADOS
SET batstart=%DATE% %TIME%
SET batuser=%USERNAME%
SET batname=%~n0
SET batext=%~x0
SET batfile=%batname%%batext%
SET batpath=%~dp0
SET battemp="%TEMP%\%batname%_%batagoraf%.tmp"
SET batpathfull="%batpath%%batname%%batext%"
FOR %%? IN (%batpathfull%) DO SET batdate=%%~t?
SET batlogname=%batname%.log
SET batlognameold=%batlogname%.old

:: Define variaveis personalizaveis
SET batspelfile=z_loader_sdm.spl
SET batspelfunction=z_loader_sdm
SET batspelmethod=%batfunction%
SET batspeldebug=0
SET batspeladminprop=user_admin
SET batspeldomsrvrprop=object_manager
SET batfiletype=CSV
SET batdebug=0
SET batlog=1
SET batfoldercomplete=PROCESSADOS
:: Fim das variaveis personalizaveis

:: Verifica os parametros
SET batargc=0
FOR %%? IN (%*) DO (
	IF [%batdebug%]==[1] ECHO %DATE% %TIME%	DEBUG		#ARG !batargc!: %%?
	SET /A batargc+=1
)
IF [%batdebug%]==[1] ECHO %DATE% %TIME%	DEBUG		Quant. de parametros informados: %batargc%
ECHO.

:: INICIO DA EXECUCAO
CALL:exibe_cabecalho
CALL:define_log
CALL:bloco_inicio

IF [%batdebug%]==[1] (
	:: Capturando variaveis de ambiente do SCRIPT
	IF DEFINED batname (
		CALL:logmsg DEBUG "Capturando variaveis utilizadas pelo SCRIPT..."
		FOR /F "tokens=1-2 delims==" %%a IN ('SET bat') DO (
			IF [%batlog%]==[1] >>%batlogpath% ECHO %DATE% %TIME%	DEBUG		%%a=%%b
		)
	) ELSE (
		CALL:logmsg WARNING "AVISO: As variaveis de ambiente do script nao puderam ser coletadas."
	)
)

:: Verifica a existencia de arquivos que precisam ser processados na pasta do script
CALL:logmsg DEBUG "Verificando arquivos %batfiletype% na pasta atual..."
SET batcountfiles=0
FOR %%a IN (*.%batfiletype%) DO SET /A batcountfiles += 1

:: Substitui a contrabarra por barra no caminho do bath
SET batfilepathfull=%batpath:\=/%

IF NOT %batcountfiles%==0 (
	CALL:logmsg DEBUG "Quant. de arquivos %batfiletype% encontrados: %batcountfiles%"
	FOR /F "" %%a IN ('DIR /B /AA /ON *.%batfiletype%') DO (
		SET batfilefull=%batfilepathfull%%%a
		FOR %%? IN (!batfilefull!) DO SET batfileproperties=%%~n?.properties
		FOR /F "tokens=2 delims==" %%a IN ('TYPE !batfileproperties! ^|find "%batspeladminprop%"') DO SET batadmin=%%a
		IF [!batadmin!]==[] SET batadmin=servicedesk

		FOR /F "tokens=2 delims==" %%a IN ('TYPE !batfileproperties! ^|find "%batspeldomsrvrprop%"') DO SET batdomsrvr=%%a
		IF [!batdomsrvr!]==[] SET batdomsrvr=domsrvr

		CALL:logmsg SIGNIFICANT "Processando o arquivo '%%a'..."
		IF [%batlog%]==[1] (
			CALL bop_cmd -d !batdomsrvr! -u !batadmin! -f %batspelfile% "%batspelfunction%('!batfilefull!', %batspeldebug%, %batspelmethod%)">>%batlogpath%
		) ELSE (
			CALL bop_cmd -d !batdomsrvr! -u !batadmin! -f %batspelfile% "%batspelfunction%('!batfilefull!', %batspeldebug%, %batspelmethod%)"
		)
		
		FOR %%? IN (!batfilefull!) DO SET batfileerror=%%~n?.error
		IF EXIST !batfileerror! (
			FOR /F "tokens=*" %%e IN ('TYPE "!batfileerror!"') DO SET baterrormsg=%%e
			DEL "!batfileerror!"
			CALL:erro1 "!baterrormsg!"
		) ELSE (
			CALL:logmsg DEBUG "O arquivo '!batfilefull!' foi processado com SUCESSO. Movendo para a pasta %batfoldercomplete%..."
			IF NOT EXIST %batfoldercomplete%\NUL MD %batfoldercomplete%
			MOVE %%a %batfoldercomplete%\ >NUL
		)
	)

) ELSE (
	CALL:logmsg SIGNIFICANT "AVISO: NENHUM arquivo %batfiletype% foi encontrado."
)

:: Caso a rotina nao tenha gerado nenhum erro, encerra a execucao
IF "!baterrormsg!"=="" (
	GOTO sucesso
) ELSE (
	GOTO falha
)
GOTO:EOF

:atualiza_data_hora
:: Consultando data atual
FOR %%A IN (%DATE%) DO (
    FOR /F "tokens=1-3 delims=/-" %%B in ("%%~A") DO (
        SET batdataf=%%D%%C%%B
        SET batdatas=%%B/%%C/%%D
    )
)
:: Consultando hora atual
FOR /F "tokens=1-3 delims=:.," %%A IN ("%TIME%") DO (
	SET bathoraf=%%A%%B%%C
	SET bathoras=%%A:%%B:%%C
)
:: Concatenando data e hora
SET batagoraf=%batdataf%%bathoraf%
SET batagoras=%batdatas% %bathoras%
GOTO:EOF

:exibe_cabecalho
:: Exibe o cabecalho do programa
ECHO *************************************************************
ECHO %batfunction% %batversion% [%batfile%]
ECHO Desenvolvido por %batauthor% em %batdate%
ECHO Descricao: %batdescription%
ECHO *************************************************************
GOTO:EOF

:logmsg
SET batmsgtype=%1
SET batmsgtext=%2
:: Removendo aspas do inicio e fim da string
SET "batmsgtype=%batmsgtype:*"=%
SET "batmsgtext=%batmsgtext:*"=%
SET batmsg=%DATE% %TIME%	%batmsgtype%		%batmsgtext%
IF "%batmsgtype%"=="SIGNIFICANT" SET batmsg=%DATE% %TIME%	%batmsgtype%	%batmsgtext%

IF [%batdebug%]==[1] (
	ECHO %batmsg%
	IF "%batmsgtype%"=="DEBUG" (
		IF [%batlog%]==[1] >>%batlogpath% ECHO %batmsg%
	)

) ELSE (
	IF NOT "%batmsgtype%"=="DEBUG" ECHO %batmsg%
)

IF [%batlog%]==[1] (
	IF NOT "%batmsgtype%"=="DEBUG" >>%batlogpath% ECHO %batmsg%

) ELSE (
	REM Registra em log sempre todos os tipos de erro
	IF "%batmsgtype%"=="ERROR" (
		>>%batlogpath% ECHO %batmsg%
	)
)
GOTO:EOF

:errofile
SET baterror_code=%ERRORLEVEL%
SET baterror_msg=%1
SET baterror_msg=%baterror_msg:"='%
CALL:logmsg ERROR "ERRO [%baterror_code%] ao processar o arquivo '%baterror_msg%'."
EXIT /B %baterror_code%
GOTO:EOF

:erro1
SET baterror_code=1
SET baterror_msg=%1
SET baterror_msg=%baterror_msg:"='%
CALL:logmsg ERROR "!baterror_msg!"
GOTO:EOF

:sucesso
CALL:logmsg SIGNIFICANT	"Operacao CONCLUIDA com sucesso."
GOTO fim

:falha
CALL:logmsg WARNING	"AVISO: Ocorreram erros durante a execucao da rotina."
GOTO fim

:cancelar
CALL:logmsg SIGNIFICANT	"Operacao CANCELADA pelo usuario."
GOTO fim

:define_log
:: Define os arquivos que serao utilizados para registros de log
IF DEFINED NX_ROOT (
	IF [%batdebug%]==[1] ECHO %DATE% %TIME%	DEBUG		Definindo local para armazenamento de logs na SDM...
	SET batlogpath="%NX_ROOT%\log\!batlogname!"
	SET batlogpathold="%NX_ROOT%\log\!batlognameold!"
	:: Substitui a barra por contrabarra no path do servicedesk
	SET batfindbar=/
	SET batreplacebar=\
	CALL SET batlogpath=%%batlogpath:!batfindbar!=!batreplacebar!%%
	CALL SET batlogpathold=%%batlogpathold:!batfindbar!=!batreplacebar!%%
) ELSE (
	IF [%batdebug%]==[1] ECHO %DATE% %TIME%	DEBUG		Definindo local para armazenamento de logs na pasta do script...
	SET batlogpath="!batpath!!batlogname!"
	SET batlogpathold="!batpath!!batlognameold!"
)
CALL:logmsg DEBUG "Arquivo de logs utilizado: %batlogpath%"
IF DEFINED NX_LOGFILE_LIMIT (
	SET batlogsizemax=%NX_LOGFILE_LIMIT%
) ELSE (
	SET batlogsizemax=5000000
)
CALL:logmsg DEBUG "Tamanho maximo do log: %batlogsizemax% bytes."
IF EXIST %batlogpath% CALL:verifica_log
GOTO:EOF

:verifica_log
:: Verifica se o arquivo de log atingiu o limite de tamanho definido
:: E em caso positivo, rotaciona o log.
CALL:logmsg DEBUG "Verificando tamanho do arquivo de log..."
FOR /F "usebackq" %%A IN ('%batlogpath%') DO SET batlogsize=%%~zA
CALL:logmsg DEBUG "Tamanho atual do arquivo de log: %batlogsize%"
IF %batlogsize% GEQ %batlogsizemax% (
	CALL:logmsg DEBUG "O arquivo de log ATINGIU o limite de tamanho de %batlogsizemax% bytes."
	CALL:logmsg DEBUG "Removendo logs antigos e rotacionando log..."
	IF EXIST "%batlognameold%" DEL "%batlognameold%"
	IF EXIST "%batlogname%" IF NOT EXIST "%batlognameold%" REN "%batlogname%" "%batlognameold%"
)
GOTO:EOF

:bloco_inicio
:: Exibe o bloco de inicio do programa
CALL:logmsg DEBUG "INICIO"
IF DEFINED batparameters (
	CALL:logmsg DEBUG "Linha de comando utilizada: %CMDCMDLINE%"
)
GOTO:EOF

:bloco_fim
:: Exibe o bloco de fim do programa
CALL:logmsg DEBUG "FIM"
IF [%batlog%]==[1] >>%batlogpath% ECHO.
GOTO:EOF

:fim
:: Reseta variaveis
CALL:bloco_fim
POPD
ENDLOCAL